using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using VirtualLab.Programming;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Functions;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Numerics.Region2D;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.Propagation;

namespace OwnCode {
    public class VLModule : IVLModule {
        
        // This module computes the standard deviation between 
        // two complex amplitude fields. The user can select 
        // both of them, first the reference, then the one to 
        // be compared against the reference. 
            
        // It is only applicable to globally polarized fields
        // with the same Jones' vector.
        
        // Please note that there exists an in-built function in 
        // VirtualLab Fusion for the calculation of the standard
        // deviation, ComplexAmplitudeOperations.ComputeDeviation.
        // This module serves as a programming example. 
        public void Run() {
            
            // Select reference field: 
            int caReference_index; // The index that identifies the field in question. 
            ComplexAmplitude caReference = (ComplexAmplitude) (Globals.ActiveDocumentHistory.BrowseLastDocuments(
                DocumentFilter.ComplexAmplitude, out caReference_index, "Reference field"));
            
            // Select field to be anayzed: 
            int caAnalysis_index; // The index that identifies the field in question. 
            ComplexAmplitude caAnalysis = (ComplexAmplitude) (Globals.ActiveDocumentHistory.BrowseLastDocuments(
                DocumentFilter.ComplexAmplitude, out caAnalysis_index, "Field for analysis"));
            
            // Compute the deviation (employing a custom function 
            // defined below): 
            double refractiveIndex = 1.0; 
            double deviation = ComputeDeviation(caReference, caAnalysis, refractiveIndex);
            
            // Display the result: 
            VL_GUI.WriteLineToMessagesTab("Relative standard deviation between the two selected fields: " + 
                deviation.ToString()); 
        }
        
        /// <summary>
        /// Function to compute the standard deviation between two 
        /// complex amplitudes.
        /// </summary>
        /// <param name="caReference"> The reference field. Must be globally polarized 
        /// and have the same Jones' vector as the field for analysis. </param>
        /// <param name="caAnalysis"> The field for analysis. Must be globally polarized 
        /// and have the same Jones' vector as caReference. </param>
        /// <param name="refractiveIndex"> The refractive index of the embedding 
        /// medium. </param>
        /// <returns> The relative standard deviation between the two fields. </returns>
        private double ComputeDeviation(
            ComplexAmplitude caReference, 
            ComplexAmplitude caAnalysis, 
            Complex refractiveIndex)
        {
            // Declare output and intermediate functions: 
            double sumOfSquareOfDifference = double.PositiveInfinity;
            double standardDeviation = double.PositiveInfinity;
            
            try
            {
                // Compute the pointwise difference between the two fields: 
                ComplexAmplitude caDifference = 
                    ComplexAmplitude.SubtractWithResampling(
                    caAnalysis, 
                    caReference, 
                    InterpolationMethod.SincFFT, 
                    InterpolationMethod.SincFFT);
            
                // Compute the sum of the square of the difference (first
                // step towards calculating the discrete equivalent of the
                // integral): 
                sumOfSquareOfDifference = ComplexFieldEvaluation.GetSum(
                    caDifference.Field, 
                    ComplexPart.ExtractSquaredAmplitude, 
                    false).Abs();
            
                // ... and scale it by multiplying by the sampling distance, 
                // with allowances for one-dimensional fields (yields 
                // the discrete equivalent of the integral): 
                if (caDifference.SamplingPoints.X > 1)
                {
                    sumOfSquareOfDifference *= caDifference.SamplingDistance.X; 
                }
                if (caDifference.SamplingPoints.Y > 1)
                {
                    sumOfSquareOfDifference *= caDifference.SamplingDistance.Y; 
                }
            
                // Next, separately, compute the square of the norm of the
                // reference field (ditto): 
                double normOfReferenceField = ComplexFieldEvaluation.GetSum(
                    caReference.Field, 
                    ComplexPart.ExtractSquaredAmplitude, 
                    false).Abs();
            
                // ... and, again, scale by multiplying by the sampling distance
                // (ditto): 
                if (caReference.SamplingPoints.X > 1)
                {
                    normOfReferenceField *= caReference.SamplingDistance.X; 
                }
                if (caReference.SamplingPoints.Y > 1)
                {
                    normOfReferenceField *= caReference.SamplingDistance.Y; 
                }
            
                // Finally, the relative deviation is given by the ratio of the 
                // integral of the square of the norm of the pointwise difference
                // between the two fields, and the integral of the square of the 
                // norm of the reference field: 
                standardDeviation = sumOfSquareOfDifference / normOfReferenceField;
            }
            catch
            {
                ;
            }
            
            // Return result: 
            return standardDeviation;
        }
    }
}
         



